//    Austin is a program for updating projects kept in XML files.
//    Copyright (C) 2000  Joseph Gregorio
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef AUSTIN_PROJECT_H
#define AUSTIN_PROJECT_H

#ifndef AUSTIN_TASK
#include "Task.h"
#endif

#include <vector>
#include <string>
#include <dom/DOM_Element.hpp>
#include "DirectedGraph.h"

class Project;

using namespace std;

ostream & operator<<(ostream & target, const Project & toWrite);

class Project {
 private:
	wstring 		name_;
	wstring 		description_;
	vector<Task> 	tasks_;

	DirectedGraph<Task> Project::fillGraph(bool forward);
	
 public:
	Project();
	Project(const Project & rhs);
	~Project();

	wstring	name();
	void	name(const wstring & value);

	wstring description();
	void 	description(const wstring & value);

	// Returns a null task if none was found.
	Task find(const wstring & name);

	Task & findOrCreate(const wstring & name);

	vector<Task> & tasks();

	vector<Task> tasksPartiallyOrdered();
	
	// Add in Start and Finish and add in dependencies if needed.
	void rationalize();

	void calculateStartAndFinishTimes(bool optimistic);

	void readConfig(DOM_Element project);
	void writeConfig(DOM_Element project);

};


#endif // AUSTIN_PROJECT_H










