//    Austin is a program for updating projects kept in XML files.
//    Copyright (C) 2000  Joseph Gregorio
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef AUSTIN_UTILITY_H
#define AUSTIN_UTILITY_H

#include <string>
#include <iostream>
#include <algorithm>
#include <vector>
#include <dom/DOM_Element.hpp>
#include <dom/DOM_Node.hpp>
#include <dom/DOM_Attr.hpp>


using namespace std;

#ifndef NO_PNG
class Project;
void generatePNG(Project & project);
#endif

// Convert from a wide string to a char (MBCS) string.
string 			stringMBCS(const wchar_t * value);

void          	outputContent(ostream& target, const DOMString &s);

ostream& 		operator<<(ostream& target, const DOMString& toWrite);
ostream& 		operator<<(ostream& target, const DOM_Node& toWrite);

vector<DOM_Element> collectElements(const wstring & name, DOM_Node parent);

DOM_Element		getSingleElementByName(DOM_Element element, const wstring & name);

int				getAttributeAsInt(DOM_Element element, const wstring & name, int defaultValue);
void 			setAttributeWithInt(DOM_Element element, const wstring & name, int value);

DOM_Element 	getOrCreateByTagName(DOM_Element element, const wstring & tagName);

struct HasName {
	wstring name_;
	HasName(const wstring & name) : name_(name) {}
	bool operator()(DOM_Element & task) {
		return name_ == task.getNodeName().rawBuffer();
	}
};

struct HasID {
	wstring id_;
	HasID(const wstring & id) : id_(id) {}
	bool operator()(DOM_Element & ele) {
		if (NULL != ele.getAttribute(L"id").rawBuffer()) {
			return (id_ == ele.getAttribute(L"id").rawBuffer());		
		} else {
			return false;
		}
	}
};

struct ElementToName {
	wstring operator()(DOM_Element ele) {
		return ele.getAttribute(L"id").rawBuffer();
	}
};


class StrX
{
  public:
	StrX(const XMLCh* const toTranscode, const unsigned int len = 0) : fLocalForm(0) {
        transcode(toTranscode, len);
    }

    ~StrX() {
        delete [] fLocalForm;
    }


    const char* localForm() const {
        return fLocalForm;
    }

  private:
	void transcode (const XMLCh* const toTranscode, const unsigned int len);

    char*   fLocalForm;
};

inline ostream& operator<<(ostream& target, const StrX& toDump)
{
    target << toDump.localForm();
    return target;
}


#endif












